///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _DBINIT_CPP
#define _DBINIT_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <WindowsX.H>
#include <ShellAPI.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <SQL.H>
#include <SQLExt.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../Resources/Resource.H"

#include "../../SharedClasses/SQLClass/cSQL.H"
#include "../../SharedClasses/SQLClass/cRecordSet.H"
#include "../../SharedClasses/CGetPKs/CGetPKs.H"
#include "../../SharedClasses/CDynString/CDynString.H"

#include "../CSockSrvr/CSockSrvr.H"

#include "../../SharedSource/Debug.H"
#include "../../SharedSource/NSWFL.H"
#include "../../SharedSource/Common.H"
#include "../../SharedSource/CRC32.H"

#include "Init.H"
#include "Routines.H"
#include "Console.H"
#include "DBInit.H"

#include "../Dialogs/ReplicationDlg.H"
#include "../Dialogs/MainDlg.H"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool GenerateTableCreationSQL(
	CSQL *lpCSQL, CSockSrvr *pSockSrvr, int iClient,
	const char *sDB, const char *sTable,
	char *sOutPK, unsigned int iOutPKSz,
	char *sOutSQL, unsigned int iOutSQLSz)
{
	CDynString SQL(10240);
	CDynString PKSQL(10240);

	CRecordSet rsTemp;

	char sCol[1024];
	char sSmlTmp[64];
	char sType[255];

	int iColLen = 0;
	int iColScale = 0;
	int iColPrec = 0;
	int iColStat = 0;
	int iIsPKCol = 0;
	int iTempSz = 0;
	int iLen = 0;

	lpCSQL->bThrowErrors = true;
	rsTemp.bThrowErrors = true;

	sprintf_s(SQL.Buffer, SQL.Alloc,
		"SELECT"
			"[PK].[Name] AS [PKName]"
		"FROM"
			"[%s].[%s].[SysObjects] AS [Obj],"
			"[%s].[%s].[SysObjects] AS [PK]"
		"WHERE"
			"[Obj].[Name] = '%s'"
			"AND [PK].[Parent_Obj] = [Obj].[ID]"
			"AND [PK].[xType] = 'PK' ORDER BY [Obj].[Name]",
		sDB, gsDefaultDBO, sDB, gsDefaultDBO, sTable);
	lpCSQL->Execute(SQL.Buffer, &rsTemp);
	SQL.Truncate();

	if(rsTemp.Fetch())
	{
		if(!rsTemp.sColumnEx(1, sOutPK, iOutPKSz, &iTempSz))
		{
			//fixme:
		}
	}
	else{
		//fixme:
	}

	rsTemp.Close();

	sprintf_s(SQL.Buffer, SQL.Alloc,
		"SELECT"
		" [Columns].[Name],"
		" [Types].[Name],"
		" (CASE WHEN [Types].[Name] IN ('nchar', 'nvarchar') THEN [Columns].[Prec] ELSE [Columns].[Length] END) AS [Length],"
		" [Columns].[Prec],"
		" [Columns].[xScale],"
		" (CASE"
		"	WHEN [Types].[Name] IN ('Binary', 'Char', 'nChar', 'nVarChar', 'VarBinary', 'VarChar')"
		"		THEN 1" //Requires column Length to be specified.
		"	WHEN [Types].[Name] IN ('Decimal')"
		"		THEN 2" //Requires column Precision and Scale to be specified
		"	ELSE 0"		//Column does not require Length, Precision or Scale to be specified
		" END) AS [TypeStatus],"
		" IsNull((SELECT 1 FROM [%s].[%s].[SysObjects] AS [Objects],"
		" [%s].[%s].[SysIndexes] AS [Indexes],"
		" [%s].[%s].[SysIndexKeys] AS [IndexKeys]"
		" WHERE [Objects].[xtype] = 'PK' AND [Objects].[Name] = [Indexes].[Name]"
		" AND [Indexes].[IndId] = [IndexKeys].[IndId] AND [Indexes].[ID] = [IndexKeys].[ID]"
		" AND [IndexKeys].[ColId] = [Columns].[ColId]"
		" AND [Objects].[Parent_Obj] = [Columns].[ID]"
		" AND [Objects].[Parent_Obj] = [Columns].[ID]), 0) AS [IsPK]"
		" FROM [%s].[%s].[SysColumns] AS [Columns]"
		" INNER JOIN [%s].[%s].[SysTypes] AS [Types] ON [Columns].[xType] = [Types].[xUserType]"
		" WHERE [Columns].[ID] = Object_ID('[%s].[%s].[%s]')"
		" ORDER BY [Columns].[ColOrder]", 
		sDB, gsDefaultDBO, sDB, gsDefaultDBO,
		sDB, gsDefaultDBO, sDB, gsDefaultDBO,
		sDB, gsDefaultDBO, sDB, gsDefaultDBO, sTable);
	lpCSQL->Execute(SQL.Buffer, &rsTemp);
	SQL.Truncate();
		
	sprintf_s(SQL.Buffer, SQL.Alloc,
		"CREATE TABLE [{SQLExch_TAG_DB}].[DBO].[%s] (\r\n", sTable);

	sprintf_s(PKSQL.Buffer, PKSQL.Alloc, "\tCONSTRAINT [");
	strcat_s(PKSQL.Buffer, PKSQL.Alloc, sOutPK);
	strcat_s(PKSQL.Buffer, PKSQL.Alloc, "] PRIMARY KEY CLUSTERED\r\n");
	strcat_s(PKSQL.Buffer, PKSQL.Alloc, "\t(\r\n");

	while(rsTemp.Fetch())
	{
		//Get the column name, save it in sCol[]
		rsTemp.sColumnEx(1, sCol, sizeof(sCol), &iTempSz);

		//Get the type name, save it in sType[]
		rsTemp.sColumnEx(2, sType, sizeof(sType), &iTempSz);

		//Get the column length.
		iColLen = rsTemp.lColumn(3);

		//Get the column precision.
		iColPrec = rsTemp.lColumn(4);

		//Get the column precision.
		iColScale = rsTemp.lColumn(5);

		//Get the column type status.
		iColStat = rsTemp.lColumn(6);

		//Get the Is PK column flag.
		iIsPKCol = rsTemp.lColumn(7);

		strcat_s(SQL.Buffer, SQL.Alloc, "\t[");
		strcat_s(SQL.Buffer, SQL.Alloc, sCol);
		strcat_s(SQL.Buffer, SQL.Alloc, "] ");

		strcat_s(SQL.Buffer, SQL.Alloc, sType);

		if(iColStat == 1)
		{
			if(iColLen <= 0)
			{
				strcat_s(SQL.Buffer, SQL.Alloc, "(MAX)");
			}
			else{
				sprintf_s(sSmlTmp, sizeof(sSmlTmp), " (%d)", iColLen);
				strcat_s(SQL.Buffer, SQL.Alloc, sSmlTmp);
			}
		}
		else if(iColStat == 2)
		{
			sprintf_s(sSmlTmp, sizeof(sSmlTmp), " (%d, %d)", iColPrec, iColScale);
			strcat_s(SQL.Buffer, SQL.Alloc, sSmlTmp);
		}

		if(iIsPKCol > 0)
		{
			strcat_s(PKSQL.Buffer, PKSQL.Alloc, "\t\t[");
			strcat_s(PKSQL.Buffer, PKSQL.Alloc, sCol);
			strcat_s(PKSQL.Buffer, PKSQL.Alloc, "],\r\n");
			strcat_s(SQL.Buffer, SQL.Alloc, " NOT NULL,\r\n");
		}
		else{
			strcat_s(SQL.Buffer, SQL.Alloc, " NULL,\r\n");
		}
	}

	PKSQL.Buffer[strlen(PKSQL.Buffer) - 3] = '\0';
	SQL.Buffer[strlen(SQL.Buffer) - 3] = '\0';

	strcat_s(PKSQL.Buffer, PKSQL.Alloc, "\r\n\t) ON [PRIMARY]");

	strcat_s(SQL.Buffer, SQL.Alloc, "\r\n");
	strcat_s(SQL.Buffer, SQL.Alloc, PKSQL.Buffer);
	strcat_s(SQL.Buffer, SQL.Alloc, "\r\n");

	strcat_s(SQL.Buffer, SQL.Alloc, ") ON [PRIMARY]\r\n");

	rsTemp.Close();

	if(strlen(SQL.Buffer) > iOutSQLSz - 1)
	{
		//Fixme: Buffer too small!
	}
	
	strcpy_s(sOutSQL, iOutSQLSz, SQL.Buffer);

	SQL.Destroy();
	PKSQL.Destroy();

	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool GenerateDBCreationSQL(CSQL *lpCSQL, CSockSrvr *pSockSrvr, int iClient, char *sTargetFile)
{
	//WriteCon("GenerateDBCreationSQL\n");

	char sDB[1024];
	char sText[1024];
	char sSQL[10240];
	char sTable[1024];
	char sPKName[1024];
	int iTempSz = 0;
	int iLen = 0;

	CRecordSet rsTables;
	CRecordSet rsDatabases;

	FILE *hTarget = NULL;

	if(fopen_s(&hTarget, sTargetFile, "wb") != 0)
	{
		return false;
	}

	lpCSQL->bThrowErrors = true;
	rsDatabases.bThrowErrors = true;

	strcpy_s(sSQL, sizeof(sSQL), "SELECT [Name] FROM [SysDatabases]"
		" WHERE [Name] NOT LIKE 'SQLExch_%'"
		" AND [Name] NOT IN ('Master', 'Model', 'MSDB', 'TempDB')"
		" ORDER BY [Name]");
	lpCSQL->Execute(sSQL, &rsDatabases);	

	while(rsDatabases.Fetch())
	{
		rsDatabases.sColumnEx(1, sDB, sizeof(sDB), &iTempSz);

		sprintf_s(sSQL, sizeof(sSQL), "USE [%s]", sDB);
		if(!lpCSQL->ExecuteNonQuery(sSQL))
		{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Failed to change client database context. Performance will be affected.", EVENT_WARN);
		}

		//Create a record set containing all of the table names that contain SQLExch_Triggers
		sprintf_s(sSQL, sizeof(sSQL),
			"SELECT"
				"[Obj].[Name] AS [TableName]"
			"FROM"
				"[%s].[%s].[SysObjects] AS [Obj]"
			"WHERE"
				"[Obj].[xType] = 'U' AND"
				"("
					"LEFT([Obj].[NAME], 8) <> 'SQLExch_'"
					"AND LEFT([Obj].[NAME], 3) <> 'SYS'"
					"AND [Obj].[ID] IN"
					"("
						"SELECT"
							"[Parent_Obj]"
						"FROM"
							"[%s].[%s].[SysObjects] AS [A]"
						"WHERE"
							"[A].[xType] = 'TR'"
							"AND LEFT([A].[NAME], 8) = 'SQLExch_'"
					")"
				")"
			"ORDER BY [Obj].[Name]",
			sDB, gsDefaultDBO, sDB, gsDefaultDBO);

		lpCSQL->Execute(sSQL, &rsTables);

		fwrite(&rsTables.RowCount, sizeof(rsTables.RowCount), 1, hTarget);

		//Loop through all of the tables that have triggers.
		while(rsTables.Fetch())
		{
			//Get the table name, save it in sTable[]
			rsTables.sColumnEx(1, sTable, sizeof(sTable), &iTempSz);

			sprintf_s(sText, sizeof(sText), "Generating SQL for %s.%s", sDB, sTable);
			WriteLogEx(pSockSrvr->icClientID[iClient], sText, EVENT_NONE);

			if(!GenerateTableCreationSQL(
				lpCSQL, pSockSrvr, iClient, sDB, sTable,
				sPKName, sizeof(sPKName), sSQL, sizeof(sSQL)))
			{
			}

			//Write Database Name.
			iLen = strlen(sDB);
			fwrite(&iLen, sizeof(iLen), 1, hTarget);
			fwrite(sDB, sizeof(char), iLen, hTarget);

			//Write Table Name.
			iLen = strlen(sTable);
			fwrite(&iLen, sizeof(iLen), 1, hTarget);
			fwrite(sTable, sizeof(char), iLen, hTarget);

			//Write PK Name.
			iLen = strlen(sPKName);
			fwrite(&iLen, sizeof(iLen), 1, hTarget);
			fwrite(sPKName, sizeof(char), iLen, hTarget);

			//Write SQL.
			iLen = strlen(sSQL);
			fwrite(&iLen, sizeof(iLen), 1, hTarget);
			fwrite(sSQL, sizeof(char), iLen, hTarget);			

			if(!pSockSrvr->bcConnected[iClient] || pSockSrvr->bcDisconnect[iClient])
			{
				rsTables.Close();
				rsDatabases.Close();
				fclose(hTarget);
				return false;
			}
		}

		rsTables.Close();

		if(!pSockSrvr->bcConnected[iClient] || pSockSrvr->bcDisconnect[iClient])
		{
			rsDatabases.Close();
			fclose(hTarget);
			return false;
		}
	}	

	rsDatabases.Close();
	fclose(hTarget);

	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
