///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _INIT_CPP
#define _INIT_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <WindowsX.H>
#include <ShellAPI.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <CommCtrl.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../Resources/Resource.H"

#include "../../SharedSource/Debug.H"
#include "../../SharedSource/NSWFL.H"
#include "../../SharedSource/Common.H"
#include "../../SharedSource/EventLog.H"
#include "../../SharedSource/CRC32.H"
#include "../../SharedSource/NTFibers.H"

#include "Init.H"
#include "Entry.H"
#include "Routines.H"

#include "../CSockSrvr/CSockSrvr.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool InitializeApp(void)
{
    LoadString(ghAppInstance, IDS_APPCOPYRIGHT, gsAppCopyRight, sizeof(gsAppCopyRight));
    LoadString(ghAppInstance, IDS_FILEVERSION, gsFileVersion, sizeof(gsFileVersion));
    LoadString(ghAppInstance, IDS_TITLECAPTION, gsTitleCaption, sizeof(gsTitleCaption));
    LoadString(ghAppInstance, IDS_REGISTRYKEY, gsRegistryKey, sizeof(gsRegistryKey));
    LoadString(ghAppInstance, IDS_EVENTNAME, gsEventName, sizeof(gsEventName));

	GetBuildArchitecture(gsRegistryKey, sizeof(gsRegistryKey), gsRegistryKey);
	GetBuildArchitecture(gsEventName, sizeof(gsEventName), gsEventName);

	InitializeCriticalSection(&csTransPerSecond);
	InitializeCriticalSection(&csLogFile);

	if(!gbExcludeInit)
	{
		if(IsApplicationRunning())
		{
			return false;
		}

		//Set SessionStartTime in registry.
		FILETIME CreationTimeFT;
		FILETIME DummyFT;
		char sRegKey[255];

		sprintf_s(sRegKey, sizeof(sRegKey), "%s\\Statistics", gsRegistryKey);

		memset(&CreationTimeFT, 0, sizeof(CreationTimeFT));

		if(GetProcessTimes(GetCurrentProcess(), &CreationTimeFT, &DummyFT, &DummyFT, &DummyFT))
		{
			SYSTEMTIME CreationTimeST;

			memset(&CreationTimeST, 0, sizeof(CreationTimeST));

			if(FileTimeToSystemTime(&CreationTimeFT, &CreationTimeST))
			{
				char sTime[100];
				char sDate[100];
				char sStartTime[200];

				GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &CreationTimeST, NULL, sDate, 64);
				GetTimeFormat(LOCALE_USER_DEFAULT, 0, &CreationTimeST, NULL, sTime, 64);

				sprintf_s(sStartTime, sizeof(sStartTime), "%s %s", sDate, sTime);

				Set_StringRegistryValue(HKEY_LOCAL_MACHINE, sRegKey, "SessionStartTime", sStartTime);
			}
		}
	}

	memset(&gMem, 0, sizeof(gMem));

	if(!gMem.Init())
	{
        MessageBox(NULL, "Failed to Initialize the Memory Pool.", gsTitleCaption, MB_TASKMODAL|MB_ICONERROR);
		return false;
	}

    INITCOMMONCONTROLSEX ICCEX;
    ICCEX.dwSize = sizeof(ICCEX);
    ICCEX.dwICC  = ICC_DATE_CLASSES;
    InitCommonControlsEx(&ICCEX);

	InitializeCRCTable();

	memset(&MDI, 0, sizeof(MDI));
	memset(&gServer, 0, sizeof(gServer));

	if(!gSC.InitializeCryptographyEx(CRYPTKEY, strlen(CRYPTKEY), true))
	{
        MessageBox(NULL, "Failed to Initialize Cryptography Set CSockSrvr.", gsTitleCaption, MB_TASKMODAL|MB_ICONERROR);
		RecordError("Failed to Initialize Cryptography Set CSockSrvr.");
        return false;
	}

	if(!GetRegistryValues())
	{
		MessageBox(NULL, "Failed to get registry values.", gsTitleCaption, MB_TASKMODAL|MB_ICONERROR);
		RecordError("Failed to get registry values.");
        return false;
	}

	//Set the default Affinity Mask if it is not set.
	if(gdwCPUAffinityMask == 0)
	{
		DWORD dwProcessAffinityMask = 0;
		DWORD dwSystemAffinityMask = 0;

		if(!GetProcessAffinityMask(GetCurrentProcess(), &dwProcessAffinityMask, &dwSystemAffinityMask))
		{
			gdwCPUAffinityMask = -1;
		}
		else{
			gdwCPUAffinityMask = dwSystemAffinityMask;
		}
	}

	SetProcessAffinityMask(GetCurrentProcess(), gdwCPUAffinityMask);
	SetProcessPriorityBoost(GetCurrentProcess(), !gbUseProcessBoost);

	gNTFibers.Init();

	if(!gbExcludeInit)
	{
		if(!OpenLogFile())
		{
			MessageBox(NULL, "Failed to open log file.", gsTitleCaption, MB_TASKMODAL|MB_ICONERROR);
			return false;
		}

		if(!gServer.Initialize(0, gdwMaxClients, !gbPacketMode, gbDropBadPackets, SENDBUFSZ, RECVBUFSZ))
		{
			WriteSysLogEx("Failed to Initialize the CSockSrvr.", EVENT_ERROR);
			return false;
		}
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool UninitializeApp(void)
{
    SaveRegistryValues();

	if(!gbExcludeInit)
	{
		if(!gServer.UnInitialize())
		{
			WriteSysLogEx("Failed to UnInitialize the CSockSrvr", EVENT_ERROR);
			return 0;
		}

		CloseLogFile();
	}

	gSC.DestroyCryptography();

	gMem.Destroy();

	gNTFibers.Destroy();

	DeleteCriticalSection(&csTransPerSecond);
	DeleteCriticalSection(&csLogFile);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
