#ifndef _CDynString_CPP
#define _CDynString_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../CMemPool/CMemPool.H"
#include "CDynString.H"
#include "../../SharedSource/Debug.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CDynString::sPrintF(const char *sFormat, ...)
{
	char *sBuf = (char *) gMem.Allocate(sizeof(char), this->Alloc);
	va_list ArgList;
	va_start(ArgList, sFormat);

	int iSz = _vsprintf_s_l(sBuf, this->Alloc, sFormat, NULL, ArgList);

	this->AppendEx(sBuf, iSz);

	free(sBuf);

	return this->Length;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CDynString::operator char*()
{
	return this->Buffer;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CDynString::operator int()
{
	return this->Length;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CDynString::CDynString()
{
	this->AllocAhead = 0;
	this->Alloc = 0;
	this->Length = 0;

	this->Buffer = NULL;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CDynString::CDynString(int Size)
{
	CDynString();

	InitEx(Size);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CDynString::~CDynString()
{
	Assert(this->AllocAhead != 0 || this->Alloc != 0 || this->Length != 0 || this->Buffer != NULL,
		"Memory leak detected in ~CDynString.");
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CDynString::Init(void)
{
	return InitEx(DYNSTRINGCHUNKSZ);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CDynString::InitEx(const int iInAllocAhead)
{
	this->AllocAhead = iInAllocAhead;

	if((this->Buffer = (char *) gMem.Allocate(sizeof(char), this->AllocAhead + 1)) == NULL)
	{
		return false;
	}

	this->Alloc = this->AllocAhead;
	this->Length = 0;

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CDynString::Truncate(void)
{
	this->Length = 0;

	this->Buffer[0] = '\0';

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CDynString::Destroy(void)
{
	this->AllocAhead = 0;
	this->Alloc = 0;
	this->Length = 0;

	gMem.Free(this->Buffer);

	this->Buffer = NULL;

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CDynString::AppendEx(const char *sDat2Append, const int iDat2AppendSz)
{
	int iRPos = 0;

	//Do we have enough RAM allocated for the newly read data?
	if((this->Length + iDat2AppendSz) >= this->Alloc)
	{
		//Figure out how much more RAM we will need (And add a few more to it).
		this->Alloc = (this->Alloc + iDat2AppendSz + this->AllocAhead) + 1;

		//Allocate more RAM.
		this->Buffer = (char *) gMem.ReAllocate(this->Buffer, this->Alloc);
	}

	while(iRPos < iDat2AppendSz)
	{
		this->Buffer[this->Length++] = sDat2Append[iRPos++];
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CDynString::Append(const char *sDat2Append)
{
	return AppendEx(sDat2Append, strlen(sDat2Append));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CDynString::AppendI(const int iNum)
{
	char sNum[32];

	sprintf_s(sNum, sizeof(sNum), "%d", iNum);

	return Append(sNum);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CDynString::AppendFileByHandle(FILE *hSource)
{
	int iBytesRead = 0;

	char sBuffer[DYNSTRINGCHUNKSZ + 1];

	do{
		iBytesRead = fread(sBuffer, sizeof(char), DYNSTRINGCHUNKSZ, hSource);
		AppendEx(sBuffer, iBytesRead);
	} while(iBytesRead == DYNSTRINGCHUNKSZ);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CDynString::AppendFile(const char *sFileName)
{
	FILE *hSource = NULL;

	if(fopen_s(&hSource, sFileName, "rb") != 0)
	{
		return false;
	}

	bool bResult = AppendFileByHandle(hSource);

	fclose(hSource);

	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CDynString::AppendF(const float fDat2Append)
{
	char sFloat[64];

	sprintf_s(sFloat, sizeof(sFloat), "%f", fDat2Append);

	return AppendEx(sFloat, strlen(sFloat));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif

